
package org.mule.module.magento.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.magento.config.ListInventoryStockItemsMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class ListInventoryStockItemsDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public ListInventoryStockItemsDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(ListInventoryStockItemsMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        Element productIdOrSkusListElement = null;
        productIdOrSkusListElement = DomUtils.getChildElementByTagName(element, "product-id-or-skus");
        List<Element> productIdOrSkusListChilds = null;
        if (productIdOrSkusListElement!= null) {
            String productIdOrSkusRef = productIdOrSkusListElement.getAttribute("ref");
            if ((productIdOrSkusRef!= null)&&(!StringUtils.isBlank(productIdOrSkusRef))) {
                if ((!productIdOrSkusRef.startsWith(patternInfo.getPrefix()))&&(!productIdOrSkusRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("productIdOrSkus", new RuntimeBeanReference(productIdOrSkusRef));
                } else {
                    builder.addPropertyValue("productIdOrSkus", productIdOrSkusRef);
                }
            } else {
                ManagedList productIdOrSkus = new ManagedList();
                productIdOrSkusListChilds = DomUtils.getChildElementsByTagName(productIdOrSkusListElement, "product-id-or-sku");
                if (productIdOrSkusListChilds!= null) {
                    for (Element productIdOrSkusChild: productIdOrSkusListChilds) {
                        String valueRef = productIdOrSkusChild.getAttribute("value-ref");
                        if ((valueRef!= null)&&(!StringUtils.isBlank(valueRef))) {
                            productIdOrSkus.add(new RuntimeBeanReference(valueRef));
                        } else {
                            productIdOrSkus.add(productIdOrSkusChild.getTextContent());
                        }
                    }
                }
                builder.addPropertyValue("productIdOrSkus", productIdOrSkus);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
            if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                propertyValues.addPropertyValue("messageProcessors", new ManagedList());
            }
            List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
            listMessageProcessors.add(definition);
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
